from abc import ABCMeta

from django.contrib.auth.models import AbstractUser

from utils.web.django.model import BaseModel, BaseMeta


class BaseUserModel(AbstractUser, BaseModel):
    class Meta(BaseMeta):
        abstract = True



from typing import Union, Dict, Any, Callable

from django.http.request import HttpRequest
from django.http.response import HttpResponse
from rest_framework.request import Request
from rest_framework.response import Response

from ..request import WebRequest
from ..response import WebResponse

REQUEST_TYPE = Union[HttpRequest, Request]
PARAMS_TYPE = Dict[str, Any]
USER_TYPE = Dict[str, str]
VIEW_TYPE = Callable[[WebRequest, Any, Any], WebResponse]
API_VIEW_HANDLER_TYPE = Callable[[HttpRequest, Any, Any], HttpResponse]
BASE_API_TYPE = Callable[[VIEW_TYPE], API_VIEW_HANDLER_TYPE]
REST_VIEW_HANDLER_TYPE = Callable[[Request, Any, Any], Response]
REST_API_TYPE = Callable[[VIEW_TYPE], REST_VIEW_HANDLER_TYPE]



import traceback

from abc import ABCMeta
from typing import Optional, List, Dict, Any, Union, Tuple

from django.db.models import Q
from django.db.models.query import QuerySet
from django.utils.decorators import method_decorator

from rest_framework.views import APIView
from rest_framework.serializers import ReturnDict, ReturnList

from drf_yasg.openapi import Schema, Parameter
from drf_yasg.utils import swagger_auto_schema

from utils.exception import ViewException, DjangoWebException, ModelException
from utils.web.django.conf import DjangoConfig
from utils.web.django.model import BaseModel
from utils.web.django.serializer import BaseSerializer
from utils.web.django.pagination import BasePagination
from utils.web.django.request import WebRequest
from utils.web.django.response import WebResponse

from .. import rest_api
from .._types import PARAMS_TYPE
from ._swagger_base import SwaggerTypeBase


class BaseRestAPIView(APIView, metaclass=ABCMeta):
    config: DjangoConfig
    model: BaseModel
    serializer: BaseSerializer
    page_class: BasePagination

    # 删除标记，False表示逻辑删除，True表示物理删除
    is_delete: bool = False
    # 过滤连接条件
    filter_condition: str = "OR"
    # 过滤符号
    filter_symbol: str = "contains"

    page_instance: BasePagination
    is_page: bool

    request_get: Optional[List[Parameter]] = None
    request_post: Optional[Schema] = None
    request_put: Optional[Schema] = None
    request_delete: Optional[Dict[str, Union[List[Parameter], Schema, None]]] = None
    swagger_type: SwaggerTypeBase = SwaggerTypeBase(request_get=request_get, request_post=request_post,
                                                    request_put=request_put, request_delete=request_delete)
    response_get: Dict[Union[str, int], Schema] = swagger_type.response()
    response_post: Dict[Union[str, int], Schema] = swagger_type.response()
    response_put: Dict[Union[str, int], Schema] = swagger_type.response()
    response_delete: Dict[Union[str, int], Schema] = swagger_type.response()

    def create_exception(self, e: Exception) -> ViewException:
        return ViewException(str(e), traceback.format_exc())

    def show_exception(self, e: DjangoWebException):
        self.config.printf.error(e.error_detail)
        self.config.printf.warning(e.error_summary)

    def __get_filter_field(self, params: PARAMS_TYPE) -> Dict[str, Any]:
        field: List[str] = self.model.get_fields()
        params_keys: List[str] = list(params.keys())
        filter_field: List[str] = list(set(field) & set(params_keys))
        filter_mapper: Dict[str, Any] = {key: params.get(key)[0] for key in filter_field}
        return filter_mapper

    def __filter(self, params: PARAMS_TYPE) -> QuerySet[BaseModel]:
        filter_field: Dict[str, Any] = self.__get_filter_field(params)
        instance: QuerySet[BaseModel] = self.model.objects.filter(is_delete=0)
        q: Q = Q()
        q.connector = self.filter_symbol
        for key, val in filter_field.items():
            if key == "is_delete":
                q.children.append((key, val))
            else:
                q.children.append((f"{key}__{self.filter_symbol}", val))
        return instance.filter(q)

    def check(self, params: PARAMS_TYPE, require_fields: List[str]):
        for field in require_fields:
            assert field in params, ViewException(f"field name's {field} field isn't exist!")

    def before_select(self, request: WebRequest, instance: Union[QuerySet[BaseModel], BaseModel], many: bool) -> \
            Union[QuerySet[BaseModel], BaseModel]:
        return instance

    def __before_select(self, request: WebRequest, params: PARAMS_TYPE, many: bool, select_id: Optional[str]) -> \
            Union[QuerySet[BaseModel], BaseModel]:
        if many:
            instance: QuerySet[BaseModel] = self.__filter(params)
            self.is_page = True
        else:
            instance: BaseModel = self.model.get_one(select_id)
        try:
            instance: Union[QuerySet[BaseModel], BaseModel] = self.before_select(request, instance, many)
            if not many and instance.is_delete:
                raise Exception("not find!")
        except Exception as e:
            raise self.create_exception(e)
        if many and self.is_page:
            self.page_instance = self.page_class()
            instance: QuerySet[BaseModel] = self.page_instance.paginate_queryset(instance, request.request, view=self)
        return instance

    @swagger_auto_schema(manual_parameters=swagger_type.request_get, responses=response_get)
    @method_decorator(rest_api())
    def get(self, request: WebRequest, id: Optional[str] = None) -> WebResponse:
        self.config.printf.info(f"request success!\npath is {request.path}\nmethod is get\nselect id is {id}")
        params: PARAMS_TYPE = request.params
        many: bool = id is None
        try:
            instance: QuerySet[BaseModel] = self.__before_select(request, params, many, id)
        except ViewException as e:
            self.show_exception(e)
            return WebResponse(status=410, error=e)
        serializer: BaseSerializer = self.serializer(instance=instance, many=many)
        data: Union[ReturnDict, ReturnList] = serializer.data.copy()
        try:
            data = self.__selected(request, data, many)
        except ViewException as e:
            self.show_exception(e)
            return WebResponse(status=410, error=e)
        return WebResponse(data=data)

    def __selected(self, request: WebRequest, data: Union[ReturnList, ReturnDict], many: bool) -> Union[
        ReturnDict, ReturnList]:
        try:
            data: Union[ReturnList, ReturnDict] = self.selected(request, data, many)
        except Exception as e:
            raise self.create_exception(e)
        if many and self.is_page:
            data: Dict[str, Any] = self.page_instance.get_paginated_response(data)
        return data

    def selected(self, request: WebRequest, data: Union[ReturnDict, ReturnList], many: bool) -> Union[
        ReturnDict, ReturnList]:
        return data

    def before_create(self, request: WebRequest, params: Dict[str, Any]) -> Dict[str, Any]:
        return params

    def __before_create(self, request: WebRequest, params: PARAMS_TYPE) -> Dict[str, Any]:
        try:
            return self.before_create(request, params)
        except Exception as e:
            raise self.create_exception(e)

    @swagger_auto_schema(request_body=swagger_type.request_post, responses=response_post)
    @method_decorator(rest_api("post"))
    def post(self, request: WebRequest) -> WebResponse:
        self.config.printf.info(f"request success!\npath is {request.path}\nmethod is post")
        params: PARAMS_TYPE = request.params
        try:
            data: Dict[str, Any] = self.__before_create(request, params)
        except ViewException as e:
            self.show_exception(e)
            return WebResponse(status=400, error=e)
        serializer: BaseSerializer = self.serializer(data=data)
        try:
            serializer.is_valid(raise_exception=True)
        except Exception as e:
            _e = self.create_exception(e)
            self.show_exception(_e)
            return WebResponse(status=401, error=_e)
        serializer.save()
        try:
            result: ReturnDict = self.__created(request, params, serializer)
        except ViewException as e:
            self.show_exception(e)
            return WebResponse(status=410, error=e)
        return WebResponse(data=result)

    def __created(self, request: WebRequest, params: PARAMS_TYPE, serializer: BaseSerializer) -> ReturnDict:
        try:
            return self.created(request, params, serializer.data.copy())
        except Exception as e:
            raise self.create_exception(e)

    def created(self, request: WebRequest, params: Dict[str, Any], data: ReturnDict) -> ReturnDict:
        return data

    def before_update(self, request: WebRequest, params: Dict[str, Any], instance: BaseModel) -> Tuple[
        Dict[str, Any], BaseModel]:
        return params, instance

    def __before_update(self, request: WebRequest, params: PARAMS_TYPE, instance: BaseModel) -> Tuple[
        Dict[str, Any], BaseModel]:
        try:
            return self.before_update(request, params, instance)
        except Exception as e:
            raise self.create_exception(e)

    @swagger_auto_schema(request_body=swagger_type.request_put, responses=response_put)
    @method_decorator(rest_api("put"))
    def put(self, request: WebRequest, id: Optional[str] = None) -> WebResponse:
        self.config.printf.info(f"request success!\npath is {request.path}\nmethod is put\nupdate id is {id}")
        try:
            assert id, ViewException("update id value isn't null!")
        except ViewException as e:
            self.show_exception(e)
            return WebResponse(status=400, error=e)
        try:
            instance: BaseModel = self.model.get_one(id)
        except ModelException as e:
            self.show_exception(e)
            return WebResponse(status=404, error=e)
        params: PARAMS_TYPE = request.params
        try:
            params, instance = self.__before_update(request, params, instance)
        except ViewException as e:
            self.show_exception(e)
            return WebResponse(status=401, error=e)
        serializer: BaseSerializer = self.serializer(instance=instance, data=params)
        try:
            serializer.is_valid(raise_exception=True)
        except Exception as e:
            _e: ViewException = self.create_exception(e)
            self.show_exception(_e)
            return WebResponse(status=402, error=_e)
        serializer.save()
        try:
            result: ReturnDict = self.__updated(request, serializer, params)
        except ViewException as e:
            self.show_exception(e)
            return WebResponse(status=410, error=e)
        return WebResponse(data=result)

    def __updated(self, request: WebRequest, serializer: BaseSerializer, params: PARAMS_TYPE) -> ReturnDict:
        try:
            return self.updated(request, serializer.data.copy(), params)
        except Exception as e:
            raise self.create_exception(e)

    def updated(self, request: WebRequest, data: ReturnDict, params: Dict[str, Any]) -> ReturnDict:
        return data

    def before_delete(self, request: WebRequest, params: Dict[str, Any], many: bool,
                      instance: Optional[BaseModel] = None) -> Union[QuerySet[BaseModel], BaseModel]:
        return instance

    def __before_delete(self, request: WebRequest, params: PARAMS_TYPE, many: bool,
                        delete_id: Optional[str] = None) -> Union[QuerySet[BaseModel], BaseModel]:
        instance: Optional[BaseModel] = None
        if not many:
            try:
                instance = self.model.get_one(delete_id)
            except ModelException as e:
                raise self.create_exception(e)
        try:
            return self.before_delete(request, params, many, instance)
        except Exception as e:
            raise self.create_exception(e)

    @swagger_auto_schema(manual_parameters=swagger_type.request_delete["query"],
                         request_body=swagger_type.request_delete['params'],
                         responses=response_delete)
    @method_decorator(rest_api("delete"))
    def delete(self, request: WebRequest, id: Optional[str] = None) -> WebResponse:
        self.config.printf.info(f"request success!\npath is {request.path}\nmethod is delete\ndelete id is {id}")
        many: bool = id is None
        params: PARAMS_TYPE = request.params
        try:
            delete: Union[QuerySet[BaseModel], BaseModel] = self.__before_delete(request, params, many, id)
        except ViewException as e:
            self.show_exception(e)
            return WebResponse(status=404, error=e)
        serializer: BaseSerializer = self.serializer(instance=delete, many=many)
        data: Union[ReturnList, ReturnDict] = serializer.data.copy()
        delete_count: int = 0
        if many:
            for delete_item in delete:
                delete_count += 1
                if self.is_delete:
                    delete_item.delete()
                else:
                    delete_item.is_delete = True
                    delete_item.save()
        else:
            delete_count += 1
            if self.is_delete:
                delete.delete()
            else:
                delete.is_delete = True
                delete.save()
        try:
            self.__deleted(request, params, data)
        except ViewException as e:
            self.show_exception(e)
            return WebResponse(status=410, error=e)
        return WebResponse(data={"delete_count": delete_count})

    def __deleted(self, request: WebRequest, params: PARAMS_TYPE, data: Union[ReturnList, ReturnDict]):
        try:
            self.deleted(request, params, data)
        except Exception as e:
            raise self.create_exception(e)

    def deleted(self, request: WebRequest, params: Dict[str, Any], data: Union[ReturnList, ReturnDict]):
        pass



from .my_base_exception import MyBaseException
from .collection_exception import CollectionException
from .config_exception import ConfigException
from .db_exception import DBException, mysql_exception
from .web_exception import WebException
from .web_exception.django_exception import DjangoWebException, model_exception
from .web_exception.django_exception.view_exception import ViewException, response_exception

MysqlException = mysql_exception.MysqlException
ModelException = model_exception.ModelException
ResponseException = response_exception.ResponseException

__all__ = [
    "MyBaseException",
    "CollectionException",
    "ConfigException",
    "DBException",
    "MysqlException",
    "WebException",
    "DjangoWebException",
    "ModelException",
    "ViewException",
    "ResponseException"
]



import functools
import traceback

from rest_framework.request import Request
from rest_framework.response import Response

from utils.exception import ViewException
from utils.web.django.conf import DjangoConfig
from utils.web.django.request import WebRequest
from utils.web.django.response import WebResponse

from .api_util import BaseRestAPI

from .._types import REST_API_TYPE, VIEW_TYPE, REST_VIEW_HANDLER_TYPE


def rest_api(method: str = "get") -> REST_API_TYPE:
    def decorator(fun: VIEW_TYPE) -> REST_VIEW_HANDLER_TYPE:
        def handler(request: Request, *args: ..., **kwargs: ...) -> Response:
            config: DjangoConfig = request.config
            base_rest_api: BaseRestAPI = BaseRestAPI(request, method, config)
            try:
                web_request: WebRequest = base_rest_api.request_init()
            except ViewException as e:
                config.printf.warning(e.error_summary)
                config.printf.error(e.error_detail)
                return WebResponse(status=403, error=e).to_rest_response()
            try:
                web_response: WebResponse = fun(web_request, *args, **kwargs)
            except Exception as e:
                err: ViewException = ViewException(str(e), error_detail=traceback.format_exc())
                config.printf.warning(err.error_summary)
                config.printf.error(err.error_detail)
                return WebResponse(status=410, error=err).to_rest_response()
            return web_response.to_rest_response()

        return handler

    return decorator


def async_rest_api(method: str = "get"):
    def decorator(fun):
        @functools.wraps(fun)
        async def handler(request: Request, *args: ..., **kwargs: ...):
            config: DjangoConfig = request.config
            base_rest_api: BaseRestAPI = BaseRestAPI(request, method, config)
            try:
                web_request: WebRequest = await base_rest_api.async_request_init()
            except ViewException as e:
                config.printf.warning(e.error_summary)
                config.printf.error(e.error_detail)
                return WebResponse(status=403, error=e).to_rest_response()
            try:
                web_response: WebResponse = await fun(web_request, *args, **kwargs)
            except ViewException as e:
                config.printf.warning(e.error_summary)
                config.printf.error(e.error_detail)
                return WebResponse(status=410, error=e).to_rest_response()
            except Exception as e:
                _e: ViewException = ViewException(str(e), traceback.format_exc())
                config.printf.error(_e.error_detail)
                config.printf.warning(_e.error_summary)
                return WebResponse(status=410, error=_e).to_rest_response()
            return web_response.to_rest_response()

        return handler

    return decorator



from abc import ABCMeta
from typing import Dict, List

from rest_framework.pagination import PageNumberPagination

from utils.web.django.utils import DjangoUtils


class BasePageNumberPagination(PageNumberPagination, metaclass=ABCMeta):
    page_query_param: str = "pagenum"
    page_size_query_param: str = "pagesize"
    page_size: int = 10

    @classmethod
    def get_paginate_option_name(cls) -> List[str]:
        return [cls.page_query_param, cls.page_size_query_param]

    async def async_paginate_queryset(self, queryset, request, view=None):
        return await DjangoUtils.to_async_fun(self.paginate_queryset, queryset, request, view)

    async def get_paginated_data(self, data):
        return self.get_paginated_response(data)

    def get_paginated_response(self, data) -> Dict[str, any]:
        return {
            "count": self.page.paginator.count,
            "next": self.get_next_link(),
            "previous": self.get_previous_link(),
            "results": data
        }



from typing import Dict
from asgiref.sync import sync_to_async
from rest_framework.serializers import ModelSerializer

from ..model import IDModel


class BaseSerializer(ModelSerializer):
    def create(self, validated_data: Dict):
        validated_data.update({"id": IDModel.create_id()})
        return super().create(validated_data)

    async def async_save(self):
        return await sync_to_async(self.save)()



import os

from typing import Any, Optional


class PathConfig:
    __instance__: Any
    # 日志文件目录
    log_url: str
    # 数据文件目录
    data_url: Optional[str]
    # 输出文件目录
    output_url: Optional[str]
    # 测试文件目录
    test_url: Optional[str]
    # 临时文件目录
    tmp_url: Optional[str]

    @classmethod
    def load(
            cls, base_url: str, log_url: str,
            data_url: Optional[str] = None,
            output_url: Optional[str] = None,
            test_url: Optional[str] = None,
            tmp_url: Optional[str] = None,
    ):
        cls.log_url = os.path.join(base_url, log_url)
        if data_url: cls.data_url = os.path.join(base_url, data_url)
        if output_url: cls.output_url = os.path.join(base_url, output_url)
        if test_url: cls.test_url = os.path.join(base_url, test_url)
        if tmp_url: cls.tmp_url = os.path.join(base_url, tmp_url)



from typing import Optional


class MyBaseException(Exception):
    error_summary: Optional[str]
    error_detail: Optional[str]

    error_id: str = "ffe2f6c7bd8b499db440d1064103779e"

    def __init__(self, error_summary: Optional[str] = None, error_detail: Optional[str] = None, *args: ...):
        self.error_summary = error_summary
        self.error_detail = error_detail
        super().__init__(*args)

    def __str__(self) -> str:
        return f"{self.__class__.__name__}:{self.error_summary}"



from ._base_view import BaseRestAPIView
from ._async_view import AsyncAPIView
from ._async_base_view import AsyncBaseAPIView
from ._swagger_base import SwaggerTypeBase

BaseView = RestAPIView = RestView = BaseRestAPIView
AsyncView = AsyncAPIView
AsyncBaseView = AsyncBaseAPIView
SwaggerType = SwaggerTypeBase

__all__ = [
    "BaseView",
    "RestAPIView",
    "RestView",
    "BaseRestAPIView",
    "AsyncView",
    "AsyncAPIView",
    "AsyncBaseView",
    "AsyncBaseAPIView",
    "SwaggerType",
    "SwaggerTypeBase"
]



from abc import ABCMeta

from utils.web.django.conf import DjangoConfig


class DjangoCeleryConfig(DjangoConfig, metaclass=ABCMeta):
    def config_init(self, base_url: str) -> None:
        super().config_init(base_url)
        self.config_object.load_redis(**self.parser['REDIS'])

    def __celery_init(self):
        redis_url: str = self.config_object.redis_config.to_url()
        self.settings.CELERY_BROKER_URL = redis_url
        self.settings.CELERY_TIMEZONE = self.settings.TIME_ZONE
        self.settings.CELERY_ACCEPT_CONTENT = ['application/json', ]
        self.settings.CELERY_TASK_SERIALIZER = 'json'
        self.settings.CELERY_RESULT_SERIALIZER = 'json'
        self.settings.CELERY_TASK_TIME_LIMIT = 5
        self.settings.CELERY_RESULT_EXPIRES = 0
        # self.settings.CELERY_TASK_ANNOTATIONS = {'tasks.add': {'rate_limit': '10/s'}}
        self.settings.CELERY_WORKER_CONCURRENCY = 20
        self.settings.CELERY_WORKER_MAX_TASKS_PER_CHILD = 200

    def install(self):
        super().install()
        self.__celery_init()



import json

from django.http.request import HttpRequest

from utils.exception import ViewException
from utils.web.django.conf import DjangoConfig
from .._base import Base
from .._types import PARAMS_TYPE


class BaseAPIUtil(Base):
    request: HttpRequest

    def __init__(self, request: HttpRequest, method: str, config: DjangoConfig):
        super().__init__(request, method, config)
        request_method: str = request.method
        assert request_method.lower() == method.lower(), ViewException("request method isn't match")

    def parse_params(self) -> PARAMS_TYPE:
        method: str = self.method.lower()
        params: PARAMS_TYPE = {}
        if method in ("get", "delete"):
            params.update(self.request.GET.dict())
        if method in ("post", "put", "delete"):
            params.update(json.loads(self.request.body.decode("utf-8")))
        return params



import traceback

from abc import ABCMeta

from typing import Any, Optional, List
from pymysql.cursors import DictCursor
from dbutils.steady_db import SteadyDBConnection

from ._result import MysqlResult
from ._types import RES_TYPE


class MysqlCollection(metaclass=ABCMeta):
    creator: Any
    max_connections: Optional[int]
    min_cached: Optional[int]
    max_cached: Optional[int]
    max_shared: Optional[int]
    blocking: bool
    set_session: List
    ping: int
    charset: str

    def __init__(
            self,
            max_connections: Optional[int] = None,
            min_cached: Optional[int] = None,
            max_cached: Optional[int] = None,
            max_shared: Optional[int] = None,
            blocking: bool = False,
            set_session: Optional[List] = None,
            ping: int = 0,
            charset: str = "utf8"
    ):
        """
        :param creator: 链接数据库的模块
        :param max_connections: 连接池允许的最大连接数，0和None表示不限制连接数
        :param min_cached: 初始化时，链接池中至少创建的空闲的链接，0表示不创建
        :param max_cached: 链接池中最多闲置的链接，0和None不限制
        :param max_shared: 一个链接最多被重复使用的次数，None表示无限制
        :param blocking: 连接池中如果没有可用连接后，是否阻塞等待。True，等待；False，不等待然后报错
        :param set_session: 开始会话前执行的命令列表。
        :param ping: ping MySQL服务端，检查是否服务可用。
            - 0 = None = never
            - 1 = default = whenever it is requested
            - 2 = when a cursor is created
            - 4 = when a query is executed
            - 7 = always
        :param charset:数据库编码
        """
        if set_session is None: self.set_session = []
        self.max_connections = max_connections
        self.min_cached = min_cached
        self.max_cached = max_cached
        self.max_shared = max_shared
        self.blocking = blocking
        self.ping = ping
        self.charset = charset

    def execute(self, conn: SteadyDBConnection, cursor: DictCursor, sql: str, data: Any = None,
                many: bool = False) -> MysqlResult:
        try:
            if not many:
                cursor.execute(sql, data)
            else:
                cursor.executemany(sql, data)
            res: RES_TYPE = cursor.fetchall()
            conn.commit()
            if len(res) == 0:
                return MysqlResult(0)
            else:
                return MysqlResult(1, data=res)
        except Exception as e:
            traceback.print_exc()
            return MysqlResult(-1, error=str(e))
        finally:
            cursor.close()
            conn.close()



from typing import Optional, Any, List, Dict

from utils.exception import MysqlException


class MysqlResult:
    __status: int
    __data: Optional[List[Dict[str, Any]]]
    __error: Optional[str]

    def __init__(self, status: int, data: List[Dict[str, Any]] = None, error: Optional[str] = None):
        assert -1 <= status <= 1, MysqlException("params status value range from -1 to 1!")
        self.__status = status
        self.__data = data
        self.__error = error

    @property
    def status(self) -> int:
        return self.__status

    @property
    def data(self) -> List[Dict[str, Any]]:
        assert self.status == 1, MysqlException("not get select result!")
        return self.__data

    @property
    def error(self) -> Optional[str]:
        return self.__error



from .path_config import Path
from .addr_config import Mysql, Redis, Kafka, Hbase

from typing import Any


class BaseConfig:
    __instance__: Any
    base_config: Path = Path()
    mysql_config: Mysql
    redis_config: Redis
    kafka_config: Kafka
    hbase_config: Hbase

    @classmethod
    def __new__(cls, *args, **kwargs):
        if cls.__instance__ is None:
            cls.__instance__ = object.__new__(cls)
        return cls.__instance__

    @classmethod
    def load_path(cls, *args, **kwargs):
        cls.base_config.load(*args, **kwargs)

    @classmethod
    def load_mysql(cls, *args, **kwargs):
        cls.mysql_config = Mysql(*args, **kwargs)

    @classmethod
    def load_redis(cls, *args, **kwargs):
        cls.redis_config = Redis(*args, **kwargs)

    @classmethod
    def load_kafka(cls, *args, **kwargs):
        cls.kafka_config = Kafka(*args, **kwargs)

    @classmethod
    def load_hbase(cls, *args, **kwargs):
        cls.hbase_config = Hbase(*args, **kwargs)



from ._base_api import base_api, async_base_api
from ._rest_api import rest_api, async_rest_api
from ._rest_view import BaseView, AsyncView, AsyncBaseView,SwaggerType
from ._user_view import BaseUserLoginView, BaseUserModel, AsyncBaseUserLoginView

__all__ = [
    "base_api",
    "async_base_api",
    "rest_api",
    "async_rest_api",
    "BaseView",
    "AsyncView",
    "AsyncBaseView",
    "BaseUserLoginView",
    "BaseUserModel",
    "AsyncBaseUserLoginView",
    "SwaggerType"
]



from typing import Callable, Any
from asgiref.sync import sync_to_async


class DjangoUtils:
    @staticmethod
    async def to_async_fun(fun: Callable, *args, **kwargs) -> Any:
        return await sync_to_async(fun)(*args, **kwargs)



import traceback
import functools

from loguru._logger import Logger

from django.http.request import HttpRequest
from django.http.response import HttpResponse

from utils.exception import ViewException
from utils.web.django.conf import DjangoConfig
from utils.web.django.request import WebRequest
from utils.web.django.response import WebResponse

from .api_util import BaseAPIUtil

from .._types import BASE_API_TYPE, VIEW_TYPE, API_VIEW_HANDLER_TYPE


def base_api(config: DjangoConfig, method: str = "get") -> BASE_API_TYPE:
    def decorator(fun: VIEW_TYPE) -> API_VIEW_HANDLER_TYPE:
        def handler(request: HttpRequest, *args: ..., **kwargs: ...) -> HttpResponse:
            try:
                api_util: BaseAPIUtil = BaseAPIUtil(config=config, request=request, method=method)
            except ViewException as e:
                config.printf.warning(e.error_summary)
                config.printf.error(e.error_detail)
                return WebResponse(status=405, error=e).to_response()
            except Exception as e:
                view_exception: ViewException = ViewException(str(e), traceback.format_exc())
                config.printf.warning(view_exception.error_summary)
                config.printf.error(view_exception.error_detail)
                return WebResponse(status=405, error=view_exception).to_response()
            try:
                web_request: WebRequest = api_util.request_init()
            except ViewException as e:
                config.printf.warning(e.error_summary)
                config.printf.error(e.error_detail)
                return WebResponse(status=403, error=e).to_response()
            except Exception as e:
                view_exception: ViewException = ViewException(str(e), traceback.format_exc())
                config.printf.warning(view_exception.error_summary)
                config.printf.error(view_exception.error_detail)
                return WebResponse(status=403, error=view_exception).to_response()
            try:
                web_response: WebResponse = fun(web_request, *args, **kwargs)
            except ViewException as e:
                config.printf.warning(e.error_summary)
                config.printf.error(e.error_detail)
                return WebResponse(status=410, error=e).to_response()
            except Exception as e:
                view_exception: ViewException = ViewException(str(e), traceback.format_exc())
                config.printf.warning(view_exception.error_summary)
                config.printf.error(view_exception.error_detail)
                return WebResponse(status=410, error=view_exception).to_response()
            return web_response.to_response()

        return handler

    return decorator


def async_base_api(config: DjangoConfig, method: str = "get"):
    printf: Logger = config.printf

    def decorator(fun):
        @functools.wraps(fun)
        async def handler(request, *args, **kwargs):
            try:
                api_util: BaseAPIUtil = BaseAPIUtil(request, method, config=config)
            except ViewException as e:
                printf.warning(e.error_summary)
                printf.error(e.error_detail)
                return WebResponse(status=405, error=e).to_response()
            try:
                web_request: WebRequest = await api_util.async_request_init()
            except ViewException as e:
                printf.warning(e.error_summary)
                printf.error(e.error_detail)
                return WebResponse(status=403, error=e).to_response()
            try:
                web_response: WebResponse = await fun(web_request, *args, **kwargs)
            except ViewException as e:
                printf.warning(e.error_summary)
                printf.error(e.error_detail)
                return WebResponse(status=410, error=e).to_response()
            return web_response.to_response()

        return handler

    return decorator



import os

from abc import ABCMeta, abstractmethod

from typing import Any, List
from configparser import ConfigParser
from loguru import logger
from loguru._logger import Logger

from .base_config import BaseConfig


class ConfigTemplate(metaclass=ABCMeta):
    __instance__: Any = None

    __parser: ConfigParser = ConfigParser()
    __base_config: BaseConfig = BaseConfig
    __log_app: Logger = logger

    TYPE: List[str] = ["BASE"]

    @classmethod
    def __new__(cls, *args, **kwargs):
        if cls.__instance__ is None:
            cls.__instance__ = object.__new__(cls)
        return cls.__instance__

    def __init__(self, base_url: str, config_path: str):
        self.__parser_init(config_path)
        self.config_init(base_url)
        self.__log_init()
        self.mysql_init()
        self.redis_init()
        self.hbase_init()
        self.kafka_init()

    def __parser_init(self, config_path: str) -> None:
        """初始化配置文件"""
        self.__parser.read(config_path, encoding="utf-8")

    def __log_init(self):
        log_url: str = self.config_object.base_config.log_url
        log_name: str = "log_{time:%Y%m%d%H}.log"
        log_path: str = os.path.join(log_url, log_name)
        self.__log_app.add(log_path, rotation="1 hours", retention="3 days", encoding="utf-8")

    def mysql_init(self) -> None:
        pass

    def redis_init(self) -> None:
        pass

    def kafka_init(self) -> None:
        pass

    def hbase_init(self) -> None:
        pass

    @abstractmethod
    def config_init(self, base_url: str) -> None:
        self.__base_config.load_path(base_url, **self.__parser[self.TYPE[0]])

    @property
    def config_object(self) -> BaseConfig:
        return self.__base_config

    @property
    def printf(self):
        return self.__log_app

    @property
    def parser(self) -> ConfigParser:
        return self.__parser



from .config import SwaggerConfig
from ._urls import swagger_view_callable
from ._params import params, Params, query, Query, response_type, ResponseType, response_schema, ResponseSchema, \
    params_types, ParamsTypes

__all__ = [
    "SwaggerConfig",
    "swagger_view_callable",
    "params",
    "Params",
    "query",
    "Query",
    "response_type",
    "ResponseType",
    "response_schema",
    "ResponseSchema",
    "params_types",
    "ParamsTypes"
]



import asyncio

from abc import ABCMeta, abstractmethod

from typing import Optional, Callable

from django.http.request import HttpRequest
from django.http.response import HttpResponse
from django.utils.deprecation import MiddlewareMixin

from utils.web.django.conf import DjangoConfig


class BaseMiddleware(MiddlewareMixin, metaclass=ABCMeta):
    config: DjangoConfig

    @abstractmethod
    def process_request(self, request: HttpRequest) -> Optional[HttpResponse]:
        return

    def process_view(self, request: HttpRequest, callback: Callable, callback_args: ..., callback_kwargs: ...) -> \
            Optional[HttpResponse]:
        if asyncio.iscoroutinefunction(callback):
            return self.config.loop.run_until_complete(callback(request, *callback_args, **callback_kwargs))

    @abstractmethod
    def process_response(self, request: HttpRequest, response: HttpResponse) -> HttpResponse:
        return response

    @abstractmethod
    def process_exception(self, request: HttpRequest, exception: Exception) -> HttpResponse:
        return HttpResponse(str(exception))

    @abstractmethod
    def process_template_response(self, request: HttpRequest, response: HttpResponse) -> HttpResponse:
        return response



from typing import Any, Optional

from pymysql.cursors import DictCursor
from dbutils.pooled_db import PooledDB
from dbutils.steady_db import SteadyDBConnection

from ._collection import MysqlCollection
from ._result import MysqlResult


class MysqlPool(MysqlCollection):
    pool: Optional[PooledDB] = None

    def __init__(self, creator: Any, *args, **kwargs):
        super().__init__(max_connections=10, min_cached=5, max_cached=10, max_shared=10)
        if self.pool is None:
            self.pool = PooledDB(
                creator,
                self.min_cached,
                self.max_cached,
                self.max_shared,
                self.max_connections,
                self.blocking,
                setsession=self.set_session,
                ping=self.ping,
                *args, **kwargs
            )

    def running(self, sql: str, data: Any = None, many: bool = False) -> MysqlResult:
        conn: SteadyDBConnection = self.pool.connection()
        cursor: DictCursor = conn.cursor(DictCursor)
        result: MysqlResult = self.execute(conn, cursor, sql, data, many)
        return result



import jwt
import uuid
import json
import base64

from abc import ABCMeta
from typing import Optional, Dict, Any, Union, List

from django.conf import settings
from django.contrib.auth import authenticate
from django.utils.decorators import method_decorator

from rest_framework.serializers import ReturnDict

from drf_yasg.openapi import Schema
from drf_yasg.utils import swagger_auto_schema

from utils.time import compute_day
from utils.exception import ViewException
from utils.web.django.view import BaseView, rest_api, AsyncBaseView, async_rest_api
from utils.web.django.request import WebRequest
from utils.web.django.response import WebResponse
from utils.web.django.serializer import BaseSerializer
from utils.web.django.swagger import Params, ParamsTypes

from .. import SwaggerType

from ._model import BaseUserModel


class BaseUserLoginView(BaseView, metaclass=ABCMeta):
    super_ = BaseView
    require: List[str] = ["username", "password", "checkcode"]
    request_post: Schema = Params(ParamsTypes.object, require=require).params({
        "username": Params(ParamsTypes.string, title="用户名", description="用户名").params(),
        "password": Params(ParamsTypes.string, title="密码", description="密码"),
        "checkcode": Params(ParamsTypes.string, title="校验码", description="用户名_校验码_密码的base64字符串").params()
    })

    response_get = super_.swagger_type.response({
        "code": Params(ParamsTypes.string, title="登录校验码", description="登录校验码").params()
    })
    user_info_body: Dict[str, Schema] = {
        "id": Params(ParamsTypes.string, title="用户ID", description="用户ID").params(),
        "register_time": Params(ParamsTypes.string, title="注册时间", description="注册时间").params(),
        "username": Params(ParamsTypes.string, title="用户名", description="用户名").params(),
        "email": Params(ParamsTypes.string, title="邮箱", description="邮箱").params()
    }
    response_post = super_.swagger_type.response({
        "user_info": Params(ParamsTypes.object, title="用户信息", description="用户信息").params(properties=user_info_body),
        "usertoken": Params(ParamsTypes.string, title="token", description="登录令牌").params()
    })

    def dispatch(self, request, *args, **kwargs):
        request.config = self.config
        return super().dispatch(request, *args, **kwargs)

    @method_decorator(rest_api())
    def get(self, request: WebRequest, id: Optional[str] = None) -> WebResponse:
        self.config.printf.info(f"request success!\npath is {request.path}\nmethod is get\nselect id is {id}")
        user_code: str = uuid.uuid4().hex[:12]
        request.session["user_code"] = user_code
        self.config.printf.success(f"current request user code's {user_code}")
        return WebResponse(data={"code": user_code})

    @method_decorator(rest_api("post"))
    def post(self, request: WebRequest) -> WebResponse:
        self.config.printf.info(f"request success!\npath is {request.path}\nmethod is get\nselect id is {id}")
        params: Dict[str, Any] = request.params
        try:
            self.check(params, ["username", "password", "checkcode"])
        except ViewException as e:
            self.show_exception(e)
            return WebResponse(status=403, error=e)
        usercode: Optional[str] = request.session.get("user_code")
        try:
            assert usercode, ViewException("user code not in session!")
        except ViewException as e:
            self.show_exception(e)
            return WebResponse(status=403, error=e)
        username: str = params.get("username")
        password: str = params.get("password")
        checkcode: str = params.get("checkcode")
        try:
            code: str = base64.b64decode(checkcode).decode("utf-8")
        except Exception as e:
            err_: ViewException = self.create_exception(e)
            self.show_exception(err_)
            return WebResponse(status=403, error=err_)
        checkcode_: str = code.split("_")[1]
        try:
            assert usercode == checkcode_, ViewException(
                "request checkcode value and session checkcode value not match!")
        except ViewException as e:
            self.show_exception(e)
            return WebResponse(status=403, error=e)
        user: Optional[BaseUserModel] = authenticate(username=username, password=password)
        try:
            assert user, ViewException("username or password no match!")
        except ViewException as e:
            self.show_exception(e)
            return WebResponse(status=403, error=e)
        serializer: BaseSerializer = self.serializer(instance=user)
        data: ReturnDict = serializer.data.copy()
        exp: float = compute_day(7).timestamp()
        payload: Dict[str, Union[str, float]] = {"exp": exp, "user": json.dumps(data)}
        token: str = jwt.encode(payload, settings.SECRET_KEY, algorithm="HS256")
        request.session.setdefault("usertoken", token)
        data.pop("password")
        data.pop("last_login")
        data.pop("is_superuser")
        data.pop("first_name")
        data.pop("last_name")
        data.pop("is_staff")
        data.pop("is_active")
        data.pop("date_joined")
        data.pop("is_delete")
        data.pop("groups")
        data.pop("user_permissions")
        return WebResponse(data={"user_info": data, "usertoken": token})


class AsyncBaseUserLoginView(AsyncBaseView, metaclass=ABCMeta):
    super_ = AsyncBaseView
    require: List[str] = ["username", "password", "checkcode"]
    request_post: Schema = Params(ParamsTypes.object, require=require).params({
        "username": Params(ParamsTypes.string, title="用户名", description="用户名").params(),
        "password": Params(ParamsTypes.string, title="密码", description="密码"),
        "checkcode": Params(ParamsTypes.string, title="校验码", description="用户名_校验码_密码的base64字符串").params()
    })
    response_get = super_.swagger_type.response({
        "code": Params(ParamsTypes.string, title="登录校验码", description="登录校验码").params()
    })
    user_info_body: Dict[str, Schema] = {
        "id": Params(ParamsTypes.string, title="用户ID", description="用户ID").params(),
        "register_time": Params(ParamsTypes.string, title="注册时间", description="注册时间").params(),
        "username": Params(ParamsTypes.string, title="用户名", description="用户名").params(),
        "email": Params(ParamsTypes.string, title="邮箱", description="邮箱").params()
    }
    response_post = super_.swagger_type.response({
        "user_info": Params(ParamsTypes.object, title="用户信息", description="用户信息").params(properties=user_info_body),
        "usertoken": Params(ParamsTypes.string, title="token", description="登录令牌").params()
    })

    @method_decorator(async_rest_api())
    async def get(self, request: WebRequest, id: Optional[str] = None) -> WebResponse:
        self.config.printf.info(f"request success!\npath is {request.path}\nmethod is get\nselect id is {id}")
        user_code: str = uuid.uuid4().hex[:12]
        request.session["user_code"] = user_code
        self.config.printf.success(f"current request user code's {user_code}")
        return WebResponse(data={"code": user_code})

    @method_decorator(rest_api("post"))
    async def post(self, request: WebRequest) -> WebResponse:
        self.config.printf.info(f"request success!\npath is {request.path}\nmethod is get\nselect id is {id}")
        params: Dict[str, Any] = request.params
        try:
            await self.check(params, ["username", "password", "checkcode"])
        except ViewException as e:
            await self.show_exception(e)
            return WebResponse(status=403, error=e)
        usercode: Optional[str] = request.session.get("user_code")
        try:
            assert usercode, ViewException("user code not in session!")
        except ViewException as e:
            await self.show_exception(e)
            return WebResponse(status=403, error=e)
        username: str = params.get("username")
        password: str = params.get("password")
        checkcode: str = params.get("checkcode")
        try:
            code: str = base64.b64decode(checkcode).decode("utf-8")
        except Exception as e:
            err_: ViewException = await self.create_exception(e)
            await self.show_exception(err_)
            return WebResponse(status=403, error=err_)
        checkcode_: str = code.split("_")[1]
        try:
            assert usercode == checkcode_, ViewException(
                "request checkcode value and session checkcode value not match!")
        except ViewException as e:
            await self.show_exception(e)
            return WebResponse(status=403, error=e)
        user: Optional[BaseUserModel] = authenticate(username=username, password=password)
        try:
            assert user, ViewException("username or password no match!")
        except ViewException as e:
            await self.show_exception(e)
            return WebResponse(status=403, error=e)
        serializer: BaseSerializer = self.serializer(instance=user)
        data: ReturnDict = serializer.data.copy()
        exp: float = compute_day(7).timestamp()
        payload: Dict[str, Union[str, float]] = {"exp": exp, "user": json.dumps(data)}
        token: str = jwt.encode(payload, settings.SECRET_KEY, algorithm="HS256")
        request.session.setdefault("usertoken", token)
        data.pop("password")
        data.pop("last_login")
        data.pop("is_superuser")
        data.pop("first_name")
        data.pop("last_name")
        data.pop("is_staff")
        data.pop("is_active")
        data.pop("date_joined")
        data.pop("is_delete")
        data.pop("groups")
        data.pop("user_permissions")
        return WebResponse(data={"user_info": data, "usertoken": token})



import json

from django.http.response import JsonResponse, HttpResponse
from rest_framework.response import Response

from ._response_types import BODYTYPE, DATATYPE, ERRTYPE


class ResponseModel:
    status: int
    body: BODYTYPE
    data: DATATYPE
    error: ERRTYPE

    def __init__(self, status: int = 200, data: DATATYPE = None, error: ERRTYPE = None):
        errno: int = 0 if 200 <= status < 300 else -1
        errmsg: str = "success" if errno == 0 else "failed"
        self.status = status
        if data is not None:
            self.data = data
        if error is not None:
            self.error = error

        self.body = {"errno": str(errno), "errmsg": errmsg}
        if hasattr(self, "data") and self.data:
            self.body['data'] = data
        print(hasattr(self, "error") and self.error)
        if hasattr(self, "error") and self.error:
            self.body['error'] = error.error_summary

    def to_dict(self) -> BODYTYPE:
        return self.body

    def to_json(self):
        return json.dumps(self.to_dict(), ensure_ascii=False)

    def to_response(self) -> HttpResponse:
        return JsonResponse(status=self.status, data=self.body)

    def to_rest_response(self) -> Response:
        return Response(status=self.status, data=self.to_dict())



import traceback

from abc import ABCMeta
from typing import Optional, Dict, Any, Union, Tuple, List

from django.db.models.query import QuerySet, Q
from django.utils.decorators import method_decorator

from rest_framework.serializers import ReturnDict, ReturnList

from drf_yasg.openapi import Schema, Parameter
from drf_yasg.utils import swagger_auto_schema

from utils.exception import ViewException, DjangoWebException, ModelException
from utils.web.django.utils import DjangoUtils
from utils.web.django.conf import DjangoConfig
from utils.web.django.model import BaseModel
from utils.web.django.serializer import BaseSerializer
from utils.web.django.pagination import BasePagination
from utils.web.django.request import WebRequest
from utils.web.django.response import WebResponse

from .. import async_rest_api
from ._async_view import AsyncAPIView
from .._types import PARAMS_TYPE

from ._swagger_base import SwaggerTypeBase


class AsyncBaseAPIView(AsyncAPIView, metaclass=ABCMeta):
    config: DjangoConfig
    model: BaseModel
    serializer: BaseSerializer
    page_class: BasePagination

    # 删除标记，False表示逻辑删除，True表示物理删除
    is_delete: bool = False
    # 过滤连接条件
    filter_condition: str = "OR"
    # 过滤符号
    filter_symbol: str = "contains"

    page_instance: BasePagination
    is_page: bool

    request_get: Optional[List[Parameter]] = None
    request_post: Optional[Schema] = None
    request_put: Optional[Schema] = None
    request_delete: Optional[Dict[str, Union[List[Parameter], Schema, None]]] = None
    swagger_type: SwaggerTypeBase = SwaggerTypeBase(request_get=request_get, request_post=request_post,
                                                    request_put=request_put, request_delete=request_delete)

    swagger_type: SwaggerTypeBase = SwaggerTypeBase()
    response_get: Dict[Union[str, int], Schema] = swagger_type.response()
    response_post: Dict[Union[str, int], Schema] = swagger_type.response()
    response_put: Dict[Union[str, int], Schema] = swagger_type.response()
    response_delete: Dict[Union[str, int], Schema] = swagger_type.response()

    async def __get_filter_field(self, params: PARAMS_TYPE) -> Dict[str, Any]:
        field: List[str] = await self.model.async_get_field()
        params_keys: List[str] = list(params.keys())
        filter_field: List[str] = list(set(field) & set(params_keys))
        filter_mapper: Dict[str, Any] = {key: params.get(key)[0] for key in filter_field}
        return filter_mapper

    async def create_exception(self, e: Exception) -> ViewException:
        return ViewException(str(e), traceback.format_exc())

    async def show_exception(self, e: DjangoWebException):
        self.config.printf.error(e.error_detail)
        self.config.printf.warning(e.error_summary)

    async def __filter(self, params: PARAMS_TYPE) -> QuerySet[BaseModel]:
        filter_field: Dict[str, Any] = await self.__get_filter_field(params)
        instance: QuerySet[BaseModel] = await self.model.async_filter(is_delete=0)
        q: Q = Q()
        q.connector = self.filter_symbol
        for key, val in filter_field.items():
            if key == "is_delete":
                q.children.append((key, val))
            else:
                q.children.append((f"{key}__{self.filter_symbol}", val))
        return await DjangoUtils.to_async_fun(instance.filter, q)

    async def before_select(self, request: WebRequest, instance: Union[QuerySet[BaseModel], BaseModel], many: bool) -> \
            Union[QuerySet[BaseModel], BaseModel]:
        return instance

    async def __before_select(self, request: WebRequest, params: PARAMS_TYPE, many: bool, select_id: Optional[str]) -> \
            Union[QuerySet[BaseModel], BaseModel]:
        if many:
            instance: QuerySet[BaseModel] = await self.__filter(params)
            self.is_page = True
        else:
            instance: BaseModel = await self.model.async_get_one(id=select_id)
            if instance.is_delete:
                raise Exception("not find!")
        try:
            instance: Union[QuerySet[BaseModel], BaseModel] = await self.before_select(request, instance, many)
        except Exception as e:
            raise await self.create_exception(e)
        if many and self.is_page:
            self.page_instance = self.page_class()
            instance = await self.page_instance.async_paginate_queryset(instance, request.request, self)
        return instance

    @swagger_auto_schema(
        manual_parameters=swagger_type.request_get,
        responses=response_get)
    @method_decorator(async_rest_api())
    async def get(self, request: WebRequest, id: Optional[str] = None) -> WebResponse:
        self.config.printf.info(f"request success!\npath is {request.path}\nmethod is get\nselect id is {id}")
        params: PARAMS_TYPE = request.params
        many: bool = id is None
        try:
            instance: Union[QuerySet[BaseModel], BaseModel] = await self.__before_select(request, params, many, id)
        except ViewException as e:
            await self.show_exception(e)
            return WebResponse(status=400, error=e)
        serializer: BaseSerializer = self.serializer(instance=instance, many=many)
        data: Union[ReturnList, ReturnDict] = serializer.data.copy()
        try:
            result: Union[ReturnList, ReturnDict] = await self.__selected(request, data, many)
        except ViewException as e:
            await self.show_exception(e)
            return WebResponse(status=410, error=e)
        return WebResponse(data=result)

    async def __selected(self, request: WebRequest, data: Union[ReturnList, ReturnDict], many: bool) -> Union[
        ReturnDict, ReturnList]:
        try:
            data: Union[ReturnList, ReturnDict] = await self.selected(request, data, many)
        except Exception as e:
            raise await self.create_exception(e)
        if many and self.is_page:
            data: Dict[str, Any] = await self.page_instance.get_paginated_data(data)
        return data

    async def selected(self, request: WebRequest, data: Union[ReturnDict, ReturnList], many: bool) -> Union[
        ReturnDict, ReturnList]:
        return data

    async def before_create(self, request: WebRequest, params: Dict[str, Any]) -> Dict[str, Any]:
        return params

    async def __before_create(self, request: WebRequest, params: PARAMS_TYPE) -> Dict[str, Any]:
        try:
            return await self.before_create(request, params)
        except Exception as e:
            raise await self.create_exception(e)

    @swagger_auto_schema(
        request_body=swagger_type.request_post,
        responses=response_post)
    @method_decorator(async_rest_api("post"))
    async def post(self, request: WebRequest) -> WebResponse:
        self.config.printf.info(f"request success!\npath is {request.path}\nmethod is post")
        params: PARAMS_TYPE = request.params
        try:
            data: Dict[str, Any] = await self.__before_create(request, params)
        except ViewException as e:
            await self.show_exception(e)
            return WebResponse(status=400, error=e)
        serializer: BaseSerializer = self.serializer(data=data)
        try:
            await DjangoUtils.to_async_fun(serializer.is_valid, raise_exception=True)
        except Exception as e:
            _e: ViewException = await self.create_exception(e)
            await self.show_exception(_e)
            return WebResponse(status=401, error=_e)
        await serializer.async_save()
        try:
            result: ReturnDict = await self.__created(request, params, serializer)
        except ViewException as e:
            await self.show_exception(e)
            return WebResponse(status=410, error=e)
        return WebResponse(data=result)

    async def __created(self, request: WebRequest, params: PARAMS_TYPE, serializer: BaseSerializer) -> ReturnDict:
        try:
            return await self.created(request, params, serializer.data.copy())
        except Exception as e:
            raise await self.create_exception(e)

    async def created(self, request: WebRequest, params: Dict[str, Any], data: ReturnDict) -> ReturnDict:
        return data

    async def before_update(self, request: WebRequest, params: Dict[str, Any], instance: BaseModel) -> Tuple[
        Dict[str, Any], BaseModel]:
        return params, instance

    async def __before_update(self, request: WebRequest, params: PARAMS_TYPE, instance: BaseModel) -> Tuple[
        Dict[str, Any], BaseModel]:
        try:
            return await self.before_update(request, params, instance)
        except Exception as e:
            raise await self.create_exception(e)

    @swagger_auto_schema(
        request_body=swagger_type.request_put,
        responses=response_put)
    @method_decorator(async_rest_api("put"))
    async def put(self, request: WebRequest, id: Optional[str] = None):
        self.config.printf.info(f"request success!\npath is {request.path}\nmethod is put\nupdate id is {id}")
        try:
            assert id, ViewException("update id value isn't null!")
        except ViewException as e:
            await self.show_exception(e)
            return WebResponse(status=400, error=e)
        try:
            instance: BaseModel = await self.model.async_get_one(id)
        except ModelException as e:
            await self.show_exception(e)
            return WebResponse(status=404, error=e)
        params: PARAMS_TYPE = request.params
        try:
            params, instance = await self.__before_update(request, params, instance)
        except ViewException as e:
            await self.show_exception(e)
            return WebResponse(status=400, error=e)
        serializer: BaseSerializer = self.serializer(instance=instance, data=params)
        try:
            await DjangoUtils.to_async_fun(serializer.is_valid, raise_exception=True)
        except Exception as e:
            _e: ViewException = await self.create_exception(e)
            await self.show_exception(_e)
            return WebResponse(status=403, error=_e)
        await serializer.async_save()
        try:
            result: ReturnDict = await self.__updated(request, serializer, params)
        except ViewException as e:
            await self.show_exception(e)
            return WebResponse(status=410, error=e)
        return WebResponse(data=result)

    async def __updated(self, request: WebRequest, serializer: BaseSerializer, params: PARAMS_TYPE) -> ReturnDict:
        try:
            return await self.updated(request, serializer.data.copy(), params)
        except Exception as e:
            raise await self.create_exception(e)

    async def updated(self, request: WebRequest, data: ReturnDict, params: Dict[str, Any]) -> ReturnDict:
        return data

    async def before_delete(self, request: WebRequest, params: Dict[str, Any], many: bool,
                            instance: Optional[BaseModel] = None) -> Union[QuerySet[BaseModel], BaseModel]:
        return instance

    async def __before_delete(self, request: WebRequest, params: PARAMS_TYPE, many: bool,
                              delete_id: Optional[str] = None) -> Union[QuerySet[BaseModel], BaseModel]:
        instance: Optional[BaseModel] = None
        if not many:
            try:
                instance: BaseModel = await self.model.async_get_one(id=delete_id)
            except ModelException as e:
                raise e

        try:
            return await self.before_delete(request, params, many, instance)
        except Exception as e:
            raise await self.create_exception(e)

    @swagger_auto_schema(
        manual_parameters=swagger_type.request_delete['query'],
        request_body=swagger_type.request_delete['params'],
        responses=response_delete)
    @method_decorator(async_rest_api("delete"))
    async def delete(self, request: WebRequest, id: Optional[str] = None):
        self.config.printf.info(f"request success!\npath is {request.path}\nmethod is delete\ndelete id is {id}")
        params: PARAMS_TYPE = request.params
        many: bool = id is None
        try:
            instance: Union[QuerySet[BaseModel], BaseModel] = await self.__before_delete(request, params, many,
                                                                                         delete_id=id)
        except ViewException as e:
            await self.show_exception(e)
            return WebResponse(status=400)
        serializer: BaseSerializer = self.serializer(instance=instance, many=many)
        data: Union[ReturnList, ReturnDict] = serializer.data.copy()
        delete_count: int = 0
        if many:
            for item in instance:
                delete_count += 1
                if self.is_delete:
                    await item.adelete()
                else:
                    item.is_delete = True
                    await item.asave()
        else:
            delete_count += 1
            if self.is_delete:
                await instance.adelete()
            else:
                instance.is_delete = True
                await instance.asave()
        try:
            await self.__deleted(request, params, data)
        except ViewException as e:
            await self.show_exception(e)
            return WebResponse(status=410, error=e)
        return WebResponse(data={"delete_count": delete_count})

    async def __deleted(self, request: WebRequest, params: PARAMS_TYPE, data: Union[ReturnList, ReturnDict]):
        try:
            await self.deleted(request, params, data)
        except Exception as e:
            raise await self.create_exception(e)

    async def deleted(self, request: WebRequest, params: Dict[str, Any], data: Union[ReturnList, ReturnDict]):
        pass

    async def check(self, params: PARAMS_TYPE, require_fields: List[str]):
        for field in require_fields:
            assert field in params, ViewException(f"field name's {field} field isn't exist!")



from abc import abstractmethod
from typing import Dict, Any, List

from django.conf import LazySettings

from utils.config import Template


class DjangoConfig(Template):
    settings: LazySettings
    __django_mysql: Dict[str, Any] = {"ENGINE": "django.db.backends.mysql"}
    max_length: int
    __system_path: List[str]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.__django_mysql_init()
        self.__system_path = self.parser['SYSTEM']['path'].split(";")

    def __django_mysql_init(self):
        mysql_config: Dict[str, Any] = self.config_object.mysql_config.to_dict()
        database: str = mysql_config.pop("database")
        self.__django_mysql['NAME'] = database
        tmp: Dict[str, Any] = {
            key.upper(): val for key, val in mysql_config.items()
        }
        self.__django_mysql.update(tmp)

    @abstractmethod
    def config_init(self, base_url: str) -> None:
        super().config_init(base_url)
        self.config_object.load_mysql(**self.parser['MYSQL'])

    @abstractmethod
    def django_setting_init(self):
        self.settings.DATABASES['default'] = self.django_mysql
        self.settings.LANGUAGE_CODE = "zh-hans"
        self.settings.TIME_ZONE = "Asia/shanghai"
        self.settings.USE_TZ = False
        self.settings.REST_FRAMEWORK = {
            'DEFAULT_PAGINATION_CLASS': 'rest_framework.pagination.PageNumberPagination',
        }

    @property
    def django_mysql(self) -> Dict[str, Any]:
        return self.__django_mysql

    @property
    def system_path(self):
        return self.__system_path

    def install(self):
        self.django_setting_init()

    def add_app(self, app_name: str):
        if app_name not in self.settings.INSTALLED_APPS:
            self.settings.INSTALLED_APPS.append(app_name)

    def add_middleware(self, middleware: str):
        if middleware not in self.settings.MIDDLEWARE:
            self.settings.MIDDLEWARE.append(middleware)

    def add_allowed_hosts(self, item: str):
        if item not in self.settings.ALLOWED_HOSTS:
            self.settings.ALLOWED_HOSTS.append(item)



import os

os.environ['DJANGO_ALLOW_ASYNC_UNSAFE'] = 'true'

from pymysql import install_as_MySQLdb

from .conf import DjangoConfig
from .celery import CeleryConfig
from .middle import BaseMiddleware
from .pagination import BasePagination
from .request import WebRequest
from .response import WebResponse

install_as_MySQLdb()

__all__ = [
    "DjangoConfig",
    "CeleryConfig",
    "BaseMiddleware",
    "BasePagination",
    "WebRequest",
    "WebResponse",
]



import base64
import hashlib
import asyncio
import aiofiles


def async_open(encoding: str = "utf-8", *args, **kwargs):
    async def open(*args, **kwargs):
        async with aiofiles.open(*args, **kwargs) as f:
            return await f.read()

    loop = asyncio.get_event_loop()
    return loop.run_until_complete(open(*args, **kwargs)).decode(encoding=encoding)


def base64_encode(data: str, encoding: str = "utf-8") -> str:
    return base64.b64encode(data.encode(encoding=encoding)).decode(encoding=encoding)


def base64_decode(code: str, encoding: str = "utf-8") -> str:
    return base64.b64decode(code).decode(encoding=encoding)


def md5_encode(data: str, encoding: str = "utf-8") -> str:
    md5 = hashlib.md5()
    md5.update(data.encode(encoding=encoding))
    return md5.hexdigest()


def sha256_encode(data: str, encoding: str = "utf-8") -> str:
    sha256 = hashlib.sha256()
    sha256.update(data.encode(encoding=encoding))
    return sha256.hexdigest()


if __name__ == '__main__':
    data = async_open(file="../../../config/config.ini", mode="rb", encoding="utf-8")
    code = base64_encode(data)
    print(base64_decode(code))
    print(md5_encode(code))
    print(sha256_encode(code))
